@@ -1,117 +1,151 @@
 package Params::Validate;
+{
+  $Params::Validate::VERSION = '1.08';
+}
+
+use 5.008001;
 
 use strict;
+use warnings;
 
-BEGIN {
-    use Exporter;
-    use vars qw( $VERSION @ISA @EXPORT @EXPORT_OK
-        %EXPORT_TAGS %OPTIONS $options $NO_VALIDATION );
+use Exporter;
+use Module::Implementation;
+use Params::Validate::Constants;
 
-    @ISA = 'Exporter';
+use vars qw( $NO_VALIDATION %OPTIONS $options );
 
-    $VERSION = '0.93';
+our @ISA = 'Exporter';
 
-    my %tags = (
-        types => [
-            qw( SCALAR ARRAYREF HASHREF CODEREF GLOB GLOBREF
-                SCALARREF HANDLE BOOLEAN UNDEF OBJECT )
+my %tags = (
+    types => [
+        qw(
+            SCALAR
+            ARRAYREF
+            HASHREF
+            CODEREF
+            GLOB
+            GLOBREF
+            SCALARREF
+            HANDLE
+            BOOLEAN
+            UNDEF
+            OBJECT
+            )
+    ],
+);
+
+our %EXPORT_TAGS = (
+    'all' => [
+        qw( validate validate_pos validation_options validate_with ),
+        map { @{ $tags{$_} } } keys %tags
+    ],
+    %tags,
+);
+
+our @EXPORT_OK = ( @{ $EXPORT_TAGS{all} }, 'set_options' );
+our @EXPORT = qw( validate validate_pos );
+
+$NO_VALIDATION = $ENV{PERL_NO_VALIDATION};
+
+{
+    my $loader = Module::Implementation::build_loader_sub(
+        implementations => [ 'XS', 'PP' ],
+        symbols         => [
+            qw(
+                validate
+                validate_pos
+                validate_with
+                validation_options
+                set_options
+                ),
         ],
     );
 
-    %EXPORT_TAGS = (
-        'all' => [
-            qw( validate validate_pos validation_options validate_with ),
-            map { @{ $tags{$_} } } keys %tags
-        ],
-        %tags,
-    );
+    $ENV{PARAMS_VALIDATE_IMPLEMENTATION} = 'PP' if $ENV{PV_TEST_PERL};
 
-    @EXPORT_OK = ( @{ $EXPORT_TAGS{all} }, 'set_options' );
-    @EXPORT = qw( validate validate_pos );
-
-    $NO_VALIDATION = $ENV{PERL_NO_VALIDATION};
-
-    eval { require Params::ValidateXS; } unless $ENV{PV_TEST_PERL};
-
-    if ( $@ || $ENV{PV_TEST_PERL} ) {
-        require Params::ValidatePP;
-    }
+    $loader->();
 }
 
 1;
 
+# ABSTRACT: Validate method/function parameters
+
 __END__
 
+=pod
+
 =head1 NAME
 
 Params::Validate - Validate method/function parameters
 
+=head1 VERSION
+
+version 1.08
+
 =head1 SYNOPSIS
 
-  use Params::Validate qw(:all);
+    use Params::Validate qw(:all);
 
-  # takes named params (hash or hashref)
-  sub foo
-  {
-      validate( @_, { foo => 1, # mandatory
-		      bar => 0, # optional
-		    }
-	      );
-  }
+    # takes named params (hash or hashref)
+    sub foo {
+        validate(
+            @_, {
+                foo => 1,    # mandatory
+                bar => 0,    # optional
+            }
+        );
+    }
 
-  # takes positional params
-  sub bar
-  {
-      # first two are mandatory, third is optional
-      validate_pos( @_, 1, 1, 0 );
-  }
+    # takes positional params
+    sub bar {
+        # first two are mandatory, third is optional
+        validate_pos( @_, 1, 1, 0 );
+    }
 
+    sub foo2 {
+        validate(
+            @_, {
+                foo =>
+                    # specify a type
+                    { type => ARRAYREF },
+                bar =>
+                    # specify an interface
+                    { can => [ 'print', 'flush', 'frobnicate' ] },
+                baz => {
+                    type      => SCALAR,     # a scalar ...
+                                             # ... that is a plain integer ...
+                    regex     => qr/^\d+$/,
+                    callbacks => {           # ... and smaller than 90
+                        'less than 90' => sub { shift() < 90 },
+                    },
+                }
+            }
+        );
+    }
 
-  sub foo2
-  {
-      validate( @_,
-		{ foo =>
-		  # specify a type
-		  { type => ARRAYREF },
+    sub with_defaults {
+        my %p = validate(
+            @_, {
+                # required
+                foo => 1,
+                # $p{bar} will be 99 if bar is not given.  bar is now
+                # optional.
+                bar => { default => 99 }
+            }
+        );
+    }
 
-		  bar =>
-		  # specify an interface
-		  { can => [ 'print', 'flush', 'frobnicate' ] },
+    sub pos_with_defaults {
+        my @p = validate_pos( @_, 1, { default => 99 } );
+    }
 
-		  baz =>
-		  { type => SCALAR,   # a scalar ...
-               	    # ... that is a plain integer ...
-                    regex => qr/^\d+$/,
-		    callbacks =>
-		    { # ... and smaller than 90
-		      'less than 90' => sub { shift() < 90 },
-		    },
-		  }
-		}
-	      );
-  }
-
-  sub with_defaults
-  {
-       my %p = validate( @_, { foo => 1, # required
-                               # $p{bar} will be 99 if bar is not
-                               # given.  bar is now optional.
-                               bar => { default => 99 } } );
-  }
-
-  sub pos_with_defaults
-  {
-       my @p = validate_pos( @_, 1, { default => 99 } );
-  }
-
-  sub sets_options_on_call
-  {
-       my %p = validate_with
-                   ( params => \@_,
-                     spec   => { foo => { type SCALAR, default => 2 } },
-                     normalize_keys => sub { $_[0] =~ s/^-//; lc $_[0] },
-                   );
-  }
+    sub sets_options_on_call {
+        my %p = validate_with(
+            params => \@_,
+            spec   => { foo => { type => SCALAR, default => 2 } },
+            normalize_keys => sub { $_[0] =~ s/^-//; lc $_[0] },
+        );
+    }
 
 =head1 DESCRIPTION
 
@@ -159,15 +193,18 @@ or a hash reference.
 Subroutines expecting named parameters should call the C<validate()>
 subroutine like this:
 
- validate( @_, { parameter1 => validation spec,
-                 parameter2 => validation spec,
-                 ...
-               } );
+    validate(
+        @_, {
+            parameter1 => validation spec,
+            parameter2 => validation spec,
+            ...
+        }
+    );
 
 Subroutines expecting positional parameters should call the
 C<validate_pos()> subroutine like this:
 
- validate_pos( @_, { validation spec }, { validation spec } );
+    validate_pos( @_, { validation spec }, { validation spec } );
 
 =head2 Mandatory/Optional Parameters
 
@@ -176,7 +213,7 @@ others are optional, this can be done very simply.
 
 For a subroutine expecting named parameters, you would do this:
 
- validate( @_, { foo => 1, bar => 1, baz => 0 } );
+    validate( @_, { foo => 1, bar => 1, baz => 0 } );
 
 This says that the "foo" and "bar" parameters are mandatory and that
 the "baz" parameter is optional.  The presence of any other
@@ -184,13 +221,13 @@ parameters will cause an error.
 
 For a subroutine expecting positional parameters, you would do this:
 
- validate_pos( @_, 1, 1, 0, 0 );
+    validate_pos( @_, 1, 1, 0, 0 );
 
 This says that you expect at least 2 and no more than 4 parameters.
 If you have a subroutine that has a minimum number of parameters but
 can take any maximum number, you can do this:
 
- validate_pos( @_, 1, 1, (0) x (@_ - 2) );
+    validate_pos( @_, 1, 1, (0) x (@_ - 2) );
 
 This will always be valid as long as at least two parameters are
 given.  A similar construct could be used for the more complex
@@ -198,7 +235,7 @@ validation parameters described further on.
 
 Please note that this:
 
- validate_pos( @_, 1, 1, 0, 1, 1 );
+    validate_pos( @_, 1, 1, 0, 1, 1 );
 
 makes absolutely no sense, so don't do it.  Any zeros must come at the
 end of the validation specification.
@@ -238,7 +275,7 @@ This one is a bit tricky.  A glob would be something like C<*FOO>, but
 not C<\*FOO>, which is a glob reference.  It should be noted that this
 trick:
 
- my $fh = do { local *FH; };
+    my $fh = do { local *FH; };
 
 makes C<$fh> a glob, not a glob reference.  On the other hand, the
 return value from C<Symbol::gensym> is a glob reference.  Either can
@@ -278,34 +315,44 @@ handle.
 To specify that a parameter must be of a given type when using named
 parameters, do this:
 
- validate( @_, { foo => { type => SCALAR },
-                 bar => { type => HASHREF } } );
+    validate(
+        @_, {
+            foo => { type => SCALAR },
+            bar => { type => HASHREF }
+        }
+    );
 
 If a parameter can be of more than one type, just use the bitwise or
 (C<|>) operator to combine them.
 
- validate( @_, { foo => { type => GLOB | GLOBREF } );
+    validate( @_, { foo => { type => GLOB | GLOBREF } );
 
 For positional parameters, this can be specified as follows:
 
- validate_pos( @_, { type => SCALAR | ARRAYREF }, { type => CODEREF } );
+    validate_pos( @_, { type => SCALAR | ARRAYREF }, { type => CODEREF } );
 
 =head2 Interface Validation
 
 To specify that a parameter is expected to have a certain set of
 methods, we can do the following:
 
- validate( @_,
-           { foo =>
-             # just has to be able to ->bar
-             { can => 'bar' } } );
+    validate(
+        @_, {
+            foo =>
+                # just has to be able to ->bar
+                { can => 'bar' }
+        }
+    );
 
  ... or ...
 
- validate( @_,
-           { foo =>
-             # must be able to ->bar and ->print
-             { can => [ qw( bar print ) ] } } );
+    validate(
+        @_, {
+            foo =>
+                # must be able to ->bar and ->print
+                { can => [qw( bar print )] }
+        }
+    );
 
 =head2 Class Validation
 
@@ -317,16 +364,18 @@ will make your API much more flexible.
 With that said, if you want to validate that an incoming parameter
 belongs to a class (or child class) or classes, do:
 
- validate( @_,
-           { foo =>
-             { isa => 'My::Frobnicator' } } );
+    validate(
+        @_,
+        { foo => { isa => 'My::Frobnicator' } }
+    );
 
  ... or ...
 
- validate( @_,
-           { foo =>
-             { isa => [ qw( My::Frobnicator IO::Handle ) ] } } );
- # must be both, not either!
+    validate(
+        @_,
+        # must be both, not either!
+        { foo => { isa => [qw( My::Frobnicator IO::Handle )] } }
+    );
 
 =head2 Regex Validation
 
@@ -334,10 +383,10 @@ If you want to specify that a given parameter must match a specific
 regular expression, this can be done with "regex" spec key.  For
 example:
 
-
- validate( @_,
-           { foo =>
-             { regex => qr/^\d+$/ } } );
+    validate(
+        @_,
+        { foo => { regex => qr/^\d+$/ } }
+    );
 
 The value of the "regex" key may be either a string or a pre-compiled
 regex created via C<qr>.
@@ -359,28 +408,38 @@ Callbacks are specified as hash reference.  The key is an id for the
 callback (used in error messages) and the value is a subroutine
 reference, such as:
 
- validate( @_,
-           { foo =>
-             { callbacks =>
-               { 'smaller than a breadbox' => sub { shift() < $breadbox },
-                 'green or blue' =>
-                  sub { $_[0] eq 'green' || $_[0] eq 'blue' } } } );
+    validate(
+        @_, {
+            foo => {
+                callbacks => {
+                    'smaller than a breadbox' => sub { shift() < $breadbox },
+                    'green or blue' =>
+                        sub { $_[0] eq 'green' || $_[0] eq 'blue' }
+                }
+            }
+        );
 
- validate( @_,
-           { foo =>
-             { callbacks =>
-               { 'bigger than baz' => sub { $_[0] > $_[1]->{baz} } } } } );
+    validate(
+        @_, {
+            foo => {
+                callbacks => {
+                    'bigger than baz' => sub { $_[0] > $_[1]->{baz} }
+                }
+            }
+        }
+    );
 
 =head2 Untainting
 
 If you want values untainted, set the "untaint" key in a spec hashref
 to a true value, like this:
 
- my %p =
-   validate( @_, { foo =>
-                   { type => SCALAR, untaint => 1 },
-                   bar =>
-                   { type => ARRAYREF } } );
+    my %p = validate(
+        @_, {
+            foo => { type => SCALAR, untaint => 1 },
+            bar => { type => ARRAYREF }
+        }
+    );
 
 This will untaint the "foo" parameter if the parameters are valid.
 
@@ -396,14 +455,20 @@ C<Params::Validate> will only attempt to untaint the reference itself.
 If you want to specify something such as type or interface, plus the
 fact that a parameter can be optional, do this:
 
- validate( @_, { foo =>
-                 { type => SCALAR },
-                 bar =>
-                 { type => ARRAYREF, optional => 1 } } );
+    validate(
+        @_, {
+            foo => { type => SCALAR },
+            bar => { type => ARRAYREF, optional => 1 }
+        }
+    );
 
 or this for positional parameters:
 
- validate_pos( @_, { type => SCALAR }, { type => ARRAYREF, optional => 1 } );
+    validate_pos(
+        @_,
+        { type => SCALAR },
+        { type => ARRAYREF, optional => 1 }
+    );
 
 By default, parameters are assumed to be mandatory unless specified as
 optional.
@@ -413,15 +478,17 @@ optional.
 It also possible to specify that a given optional parameter depends on
 the presence of one or more other optional parameters.
 
- validate( @_, { cc_number =>
-                 { type => SCALAR, optional => 1,
-                   depends => [ 'cc_expiration', 'cc_holder_name' ],
-                 },
-                 cc_expiration
-                 { type => SCALAR, optional => 1 },
-                 cc_holder_name
-                 { type => SCALAR, optional => 1 },
-               } );
+    validate(
+        @_, {
+            cc_number => {
+                type     => SCALAR,
+                optional => 1,
+                depends  => [ 'cc_expiration', 'cc_holder_name' ],
+            },
+            cc_expiration  { type => SCALAR, optional => 1 },
+            cc_holder_name { type => SCALAR, optional => 1 },
+        }
+    );
 
 In this case, "cc_number", "cc_expiration", and "cc_holder_name" are
 all optional.  However, if "cc_number" is provided, then
@@ -443,9 +510,9 @@ declared as part of your parameter specification.
 
 =head2 Specifying defaults
 
-If the C<validate()> or C<validate_pos()> functions are called in a
-list context, they will return an array or hash containing the
-original parameters plus defaults as indicated by the validation spec.
+If the C<validate()> or C<validate_pos()> functions are called in a list
+context, they will return a hash or containing the original parameters plus
+defaults as indicated by the validation spec.
 
 If the function is not called in a list context, providing a default
 in the validation spec still indicates that the parameter is optional.
@@ -456,9 +523,9 @@ calling function.
 
 Simple examples of defaults would be:
 
- my %p = validate( @_, { foo => 1, bar => { default => 99 } } );
+    my %p = validate( @_, { foo => 1, bar => { default => 99 } } );
 
- my @p = validate( @_, 1, { default => 99 } );
+    my @p = validate_pos( @_, 1, { default => 99 } );
 
 In scalar context, a hash reference or array reference will be
 returned, as appropriate.
@@ -479,27 +546,50 @@ call, you will probably want to remove the class/object from the
 parameter list B<before> calling C<validate()> or C<validate_pos()>.
 If your method expects named parameters, then this is necessary for
 the C<validate()> function to actually work, otherwise C<@_> will not
-be useable as a hash, because it will first have your object (or
+be usable as a hash, because it will first have your object (or
 class) B<followed> by a set of keys and values.
 
 Thus the idiomatic usage of C<validate()> in a method call will look
 something like this:
 
- sub method
- {
-     my $self = shift;
+    sub method {
+        my $self = shift;
 
-     my %params = validate( @_, { foo => 1, bar => { type => ARRAYREF } } );
- }
+        my %params = validate(
+            @_, {
+                foo => 1,
+                bar => { type => ARRAYREF },
+            }
+        );
+    }
+
+=head2 Speeding Up Validation
+
+In most cases, the validation spec will remain the same for each call to a
+subroutine. In that case, you can speed up validation by defining the
+validation spec just once, rather than on each call to the subroutine:
+
+    my %spec = ( ... );
+    sub foo {
+        my %params = validate( @_, \%spec );
+    }
+
+You can also use the C<state> feature to do this:
+
+    use feature 'state';
+
+    sub foo {
+        state $spec = { ... };
+        my %params = validate( @_, $spec );
+    }
 
 =head1 "GLOBAL" OPTIONS
 
-Because the API for the C<validate()> and C<validate_pos()> functions
-does not make it possible to specify any options other than the the
-validation spec, it is possible to set some options as
-pseudo-'globals'.  These allow you to specify such things as whether
-or not the validation of named parameters should be case sensitive,
-for one example.
+Because the API for the C<validate()> and C<validate_pos()> functions does not
+make it possible to specify any options other than the validation spec, it is
+possible to set some options as pseudo-'globals'.  These allow you to specify
+such things as whether or not the validation of named parameters should be
+case sensitive, for one example.
 
 These options are called pseudo-'globals' because these settings are
 B<only applied to calls originating from the package that set the
@@ -537,27 +627,27 @@ Any alterations made by this callback will be reflected in the
 parameter hash that is returned by the validation function.  For
 example:
 
-  sub foo {
-      return
-        validate_with( params => \@_,
-                       spec   => { foo => { type => SCALAR } },
-                       normalize_keys =>
-                       sub { my $k = shift; $k =~ s/^-//; return uc $k },
-                     );
+    sub foo {
+        return validate_with(
+            params => \@_,
+            spec   => { foo => { type => SCALAR } },
+            normalize_keys =>
+                sub { my $k = shift; $k =~ s/^-//; return uc $k },
+        );
 
-  }
+    }
 
-  %p = foo( foo => 20 );
+    %p = foo( foo => 20 );
 
-  # $p{FOO} is now 20
+    # $p{FOO} is now 20
 
-  %p = foo( -fOo => 50 );
+    %p = foo( -fOo => 50 );
 
-  # $p{FOO} is now 50
+    # $p{FOO} is now 50
 
 The callback must return a defined value.
 
-If a callback is given than the deprecated "ignore_case" and
+If a callback is given then the deprecated "ignore_case" and
 "strip_leading" options are ignored.
 
 =item * allow_extra => $boolean
@@ -613,13 +703,14 @@ considered identical.
 The C<validate_with()> function can be used to set the options listed
 above on a per-invocation basis.  For example:
 
-  my %p =
-      validate_with
-          ( params => \@_,
-            spec   => { foo => { type => SCALAR },
-                        bar => { default => 10 } },
-            allow_extra => 1,
-          );
+    my %p = validate_with(
+        params => \@_,
+        spec   => {
+            foo => { type    => SCALAR },
+            bar => { default => 10 }
+        },
+        allow_extra => 1,
+    );
 
 In addition to the options listed above, it is also possible to set
 the option "called", which should be a string.  This string will be
@@ -630,23 +721,25 @@ This subroutine will validate named parameters as a hash if the "spec"
 parameter is a hash reference.  If it is an array reference, the
 parameters are assumed to be positional.
 
-  my %p =
-      validate_with
-          ( params => \@_,
-            spec   => { foo => { type => SCALAR },
-                        bar => { default => 10 } },
-            allow_extra => 1,
-            called => 'The Quux::Baz class constructor',
-          );
+    my %p = validate_with(
+        params => \@_,
+        spec   => {
+            foo => { type    => SCALAR },
+            bar => { default => 10 }
+        },
+        allow_extra => 1,
+        called      => 'The Quux::Baz class constructor',
+    );
 
-  my @p =
-      validate_with
-          ( params => \@_,
-            spec   => [ { type => SCALAR },
-                        { default => 10 } ],
-            allow_extra => 1,
-            called => 'The Quux::Baz class constructor',
-          );
+    my @p = validate_with(
+        params => \@_,
+        spec   => [
+            { type    => SCALAR },
+            { default => 10 }
+        ],
+        allow_extra => 1,
+        called      => 'The Quux::Baz class constructor',
+    );
 
 =head1 DISABLING VALIDATION
 
@@ -664,21 +757,20 @@ variable.  It is B<strongly> recommended that you B<localize> any
 changes to this variable, because other modules you are using may
 expect validation to be on when they execute.  For example:
 
+    {
+        local $Params::Validate::NO_VALIDATION = 1;
 
-  {
-      local $Params::Validate::NO_VALIDATION = 1;
-      # no error
-      foo( bar => 2 );
-  }
+        # no error
+        foo( bar => 2 );
+    }
 
-  # error
-  foo( bar => 2 );
+    # error
+    foo( bar => 2 );
 
-  sub foo
-  {
-      my %p = validate( @_, { foo => 1 } );
-      ...
-  }
+    sub foo {
+        my %p = validate( @_, { foo => 1 } );
+        ...;
+    }
 
 But if you want to shoot yourself in the foot and just turn it off, go
 ahead!
@@ -701,8 +793,6 @@ via email at bug-params-validate@rt.cpan.org.
 
 Support questions can be sent to Dave at autarch@urth.org.
 
-The code repository is at https://svn.urth.org/svn/Params-Validate/
-
 =head1 DONATIONS
 
 If you'd like to thank me for the work I've done on this module,
@@ -724,14 +814,16 @@ To donate, log into PayPal and send money to autarch@urth.org or use
 the button on this page:
 L<http://www.urth.org/~autarch/fs-donation.html>
 
-=head1 AUTHORS
+=head1 AUTHOR
 
 Dave Rolsky, <autarch@urth.org> and Ilya Martynov <ilya@martynov.org>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2004-2007 David Rolsky.  All rights reserved.  This
-program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This software is Copyright (c) 2013 by Dave Rolsky and Ilya Martynov.
+
+This is free software, licensed under:
+
+  The Artistic License 2.0 (GPL Compatible)
 
 =cut
